library(tidyverse)
library(leaflet)
library(tidygeocoder)
library(leafpop)



#- geolocalizamos el SFPIE, con el pkg tidygeocoder ----------------------------
dfa <- data.frame(adress = "Facultad de Economía", 
                 street = "Avinguda dels tarongers", 
                 city = "Valencia", 
                 postalcode = "46022",
                 country = "Spain")

dfa <- dfa %>% mutate(text_to_geocode = paste( street, city, country, sep = ", "))

dfa <- dfa %>% tidygeocoder::geocode(text_to_geocode, method = "osm")
#dfa <- dfa %>% tidygeocoder::geocode(street = street, city = city, postalcode = postalcode, country = country, method = "osm")




#- hacemos un mapa con el paquete leaflet --------------------------------------
map <- leaflet() %>%
       addTiles() %>% 
       setView(lng = dfa$long, lat = dfa$lat, zoom = 16) 
map


#- añadimos markers al mapa
map <- map %>% 
       addMarkers(lng = dfa$long, lat = dfa$lat, popup = "Facultat") %>% 
       addPopups(lng = dfa$long, lat = dfa$lat, popup = "Facultat") 
map


#- vamos a ponerle una imagen al mapa ------------------------------------------
img <-  "https://www.uv.es/recursos/fatwirepub/ccurl/12/318/P01.jpg"
img <-  "./imagenes/R-logo.png"


map2 <- map %>% 
  addCircleMarkers(lng = dfa$long, lat = dfa$lat, group = "pnt") %>%
  addPopupImages(img, group = "pnt")


map2

#- se pueden poner muuuchos más elementos: https://rstudio.github.io/leaflet/basemaps.html
#- hay muchos "providers": http://leaflet-extras.github.io/leaflet-providers/preview/
map2 %>% addProviderTiles(providers$Stamen.Toner)
map2 %>% addProviderTiles(providers$Stamen.Terrain)
map2 %>% addProviderTiles(providers$Esri.NatGeoWorldMap)



map2 %>% addProviderTiles(providers$MtbMap) #- carril bici
  
